# -*- coding: utf-8 -*-

import typing as T
import dataclasses

import sayt.api as sayt
import aws_arns.api as arns
import aws_console_url.api as acu

from .. import res_lib as rl

if T.TYPE_CHECKING:
    from ..ars_def import ARS


@dataclasses.dataclass
class CodeCommitRepository(rl.ResourceDocument):
    # fmt: off
    repo_arn: str = dataclasses.field(metadata={"field": sayt.StoredField(name="repo_arn")})
    # fmt: on

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        return cls(
            raw_data=resource,
            id=resource["repositoryName"],
            name=resource["repositoryName"],
            repo_arn=arns.res.CodeCommitRepository.new(
                aws_account_id=bsm.aws_account_id,
                aws_region=bsm.aws_region,
                name=resource["repositoryName"],
            ).to_arn(),
        )

    @property
    def title(self) -> str:
        return rl.format_key_value("repo_name", self.name)

    @property
    def autocomplete(self) -> str:
        return self.name

    @property
    def arn(self) -> str:
        return self.repo_arn

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.codecommit.get_repo(repo_or_arn=self.arn)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.codecommit.repositories

    # fmt: off
    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        from_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)
        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.codecommit_client.get_repository(repositoryName=self.name)
            dct = res["repositoryMetadata"]
            accountId = dct.get("accountId")
            repositoryId = dct.get("repositoryId")
            repositoryName = dct.get("repositoryName")
            repositoryDescription = dct.get("repositoryDescription")
            defaultBranch = dct.get("defaultBranch")
            lastModifiedDate = dct.get("lastModifiedDate")
            creationDate = dct.get("creationDate")
            cloneUrlHttp = dct.get("cloneUrlHttp")
            cloneUrlSsh = dct.get("cloneUrlSsh")
            detail_items.extend([
                from_detail("accountId", accountId, url=url),
                from_detail("repositoryId", repositoryId, url=url),
                from_detail("repositoryName", repositoryName, url=url),
                from_detail("repositoryDescription", self.one_line(repositoryDescription), url=url),
                from_detail("defaultBranch", defaultBranch, url=url),
                from_detail("lastModifiedDate", lastModifiedDate, url=url),
                from_detail("creationDate", creationDate, url=url),
                from_detail("cloneUrlHttp", cloneUrlHttp, url=url),
                from_detail("cloneUrlSsh", cloneUrlSsh, url=url),
                from_detail("cloneUrlGitRemoteCodecommit", f"codecommit::{ars.bsm.aws_region}://{self.name}", url=url)
            ])

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.codecommit_client.list_tags_for_resource(resourceArn=self.arn)
            tags = rl.extract_tags(res)
            detail_items.extend(rl.DetailItem.from_tags(tags, url))

        return detail_items
    # fmt: on


class CodeCommitRepositorySearcher(rl.BaseSearcher[CodeCommitRepository]):
    pass


codecommit_repository_searcher = CodeCommitRepositorySearcher(
    # list resources
    service="codecommit",
    method="list_repositories",
    is_paginator=True,
    default_boto_kwargs={
        "sortBy": "lastModifiedDate",
        "order": "descending",
        "PaginationConfig": {
            "MaxItems": 5000,
        },
    },
    result_path=rl.ResultPath("repositories"),
    # extract document
    doc_class=CodeCommitRepository,
    # search
    resource_type=rl.SearcherEnum.codecommit_repository.value,
    fields=CodeCommitRepository.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(
        rl.SearcherEnum.codecommit_repository.value
    ),
    more_cache_key=None,
)



# -*- coding: utf-8 -*-

import typing as T
import dataclasses

import sayt.api as sayt
import aws_arns.api as arns
import aws_console_url.api as acu

from .. import res_lib as rl

if T.TYPE_CHECKING:
    from ..ars_def import ARS

dynamodb_table_status_icon_mapper = {
    "CREATING": "🟡",
    "UPDATING": "🟡",
    "DELETING": "🔴",
    "ACTIVE": "🟢",
    "INACCESSIBLE_ENCRYPTION_CREDENTIALS": "🔴",
    "ARCHIVING": "🟡",
    "ARCHIVED": "🔴",
}


@dataclasses.dataclass
class DynamodbTable(rl.ResourceDocument):
    # fmt: off
    table_arn: str = dataclasses.field(metadata={"field": sayt.StoredField(name="table_arn")})
    # fmt: on

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        return cls(
            raw_data=resource,
            id=resource,
            name=resource,
            table_arn=arns.res.DynamodbTable.new(
                aws_account_id=bsm.aws_account_id,
                aws_region=bsm.aws_region,
                table_name=resource,
            ).to_arn(),
        )

    @property
    def title(self) -> str:
        return rl.format_key_value("table_name", self.name)

    @property
    def autocomplete(self) -> str:
        return self.name

    @property
    def arn(self) -> str:
        return self.table_arn

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.dynamodb.get_table(table_or_arn=self.arn)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.dynamodb.tables

    # fmt: off
    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        from_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.dynamodb_client.describe_table(TableName=self.name)
            dct = res["Table"]

            attrs = {d["AttributeName"]: d["AttributeType"] for d in dct.get("AttributeDefinitions", [])}
            keys = {d["AttributeName"]: d["KeyType"] for d in dct.get("KeySchema", [])}
            for attr_name, key_type in keys.items():
                data_type = attrs[attr_name]
                item = rl.DetailItem.new(
                    title="🔑 key_schema {}, {}, {}".format(
                        rl.format_key_value("key", attr_name),
                        rl.format_key_value("key_type", key_type),
                        rl.format_key_value("data_type", data_type),
                    ),
                    subtitle=f"🌐 {rl.ShortcutEnum.ENTER} to open url, 📋 {rl.ShortcutEnum.CTRL_A} to copy key name.",
                )
                detail_items.append(item)

            TableSizeBytes = dct.get("TableSizeBytes", "Unknown")
            ItemCount = dct.get("ItemCount", "Unknown")
            BillingMode = dct.get("BillingModeSummary", {}).get("BillingMode", "Unknown")
            TableClass = dct.get("TableClassSummary", {}).get("BillingMode", "Unknown")
            detail_items.extend(
                [
                    from_detail("TableSizeBytes", TableSizeBytes, url=url),
                    from_detail("ItemCount", ItemCount, url=url),
                    from_detail("BillingMode", BillingMode, url=url),
                    from_detail("TableClass", TableClass, url=url),
                ]
            )

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.dynamodb_client.list_tags_of_resource(ResourceArn=self.arn)
            tags = rl.extract_tags(res)
            detail_items.extend(rl.DetailItem.from_tags(tags, url=url))

        return detail_items
    # fmt: on


class DynamodbTableSearcher(rl.BaseSearcher[DynamodbTable]):
    pass


dynamodb_table_searcher = DynamodbTableSearcher(
    # list resources
    service="dynamodb",
    method="list_tables",
    is_paginator=True,
    default_boto_kwargs={
        "PaginationConfig": {"MaxItems": 9999, "PageSize": 100},
    },
    result_path=rl.ResultPath("TableNames"),
    # extract document
    doc_class=DynamodbTable,
    # search
    resource_type=rl.SearcherEnum.dynamodb_table.value,
    fields=DynamodbTable.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(rl.SearcherEnum.dynamodb_table.value),
    more_cache_key=None,
)



# -*- coding: utf-8 -*-

from .res_lib import format_shortcut
from .res_lib import highlight_text
from .res_lib import format_resource_type
from .res_lib import format_key
from .res_lib import format_value
from .res_lib import format_key_value
from .res_lib import ShortcutEnum
from .res_lib import SUBTITLE
from .res_lib import SHORT_SUBTITLE
from .res_lib import SearcherEnum
from .res_lib import T_RESULT_DATA
from .res_lib import ResourceIterproxy
from .res_lib import ResultPath
from .res_lib import list_resources
from .res_lib import extract_tags
from .res_lib import BaseArsDocument
from .res_lib import T_ARS_DOCUMENT
from .res_lib import ResourceTypeDocument
from .res_lib import get_utc_now
from .res_lib import to_human_readable_elapsed
from .res_lib import to_utc_dt
from .res_lib import SIMPLE_DT_FMT
from .res_lib import to_simple_dt_fmt
from .res_lib import to_iso_dt_fmt
from .res_lib import get_none_or_default
from .res_lib import get_description
from .res_lib import get_datetime
from .res_lib import get_datetime_simple_fmt
from .res_lib import get_datetime_iso_fmt
from .res_lib import ResourceDocument
from .res_lib import T_ARS_RESOURCE_DOCUMENT
from .res_lib import BaseArsItem
from .res_lib import T_ARS_ITEM
from .res_lib import DetailItem
from .res_lib import ExceptionItem
from .res_lib import FileItem
from .res_lib import InfoItem
from .res_lib import UrlItem
from .res_lib import AwsResourceTypeItem
from .res_lib import AwsResourceItem
from .res_lib import SetAwsProfileItem
from .res_lib import ShowAwsInfoItem
from .res_lib import preprocess_query
from .res_lib import BaseSearcher
from .res_lib import T_SEARCHER
from .res_lib import config

from .ars_def import ARS

from .handlers.api import search_aws_profile_handler
from .handlers.api import search_resource_type_handler
from .handlers.api import search_resource_handler
from .handlers.api import show_aws_info_handler

from .ui_def import UI
from .ui_def import handler



# -*- coding: utf-8 -*-

import typing as T
import dataclasses

import sayt.api as sayt
import aws_arns.api as arns
import aws_console_url.api as acu

from .. import res_lib as rl

if T.TYPE_CHECKING:
    from ..ars_def import ARS


ec2_instance_state_icon_mapper = {
    "pending": "🟡",
    "running": "🟢",
    "shutting-down": "🟤",
    "terminated": "⚫",
    "stopping": "🟠",
    "stopped": "🔴️",
}


class Ec2Mixin:
    @staticmethod
    def get_tags(raw_data) -> T.Dict[str, str]:
        return {dct["Key"]: dct["Value"] for dct in raw_data.get("Tags", [])}

    @property
    def tags(self: T.Union["Ec2Mixin", rl.ResourceDocument]) -> T.Dict[str, str]:
        return self.get_tags(self.raw_data)


@dataclasses.dataclass
class Ec2Instance(rl.ResourceDocument, Ec2Mixin):
    """
    todo: docstring
    """
    # fmt: off
    state: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="state", minsize=2, maxsize=4, stored=True)})
    vpc_id: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="vpc_id", minsize=2, maxsize=4, stored=True)})
    subnet_id: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="subnet_id", minsize=2, maxsize=4, stored=True)})
    id_ng: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="id_ng", minsize=2, maxsize=4, stored=True)})
    inst_arn: str = dataclasses.field(metadata={"field": sayt.StoredField(name="inst_arn")})
    # fmt: on

    @property
    def state_icon(self) -> str:
        return ec2_instance_state_icon_mapper[self.state]

    @property
    def inst_type(self) -> str:
        return self.raw_data["InstanceType"]

    @property
    def public_ip(self) -> str:
        return self.raw_data.get("PublicIpAddress", "NA")

    @property
    def private_ip(self) -> str:
        return self.raw_data.get("PrivateIpAddress", "NA")

    @property
    def platform(self) -> str:
        return self.raw_data.get("Platform", "NA")

    @property
    def inst_profile_arn(self) -> str:
        return self.raw_data.get("IamInstanceProfile", {}).get("Arn", "NA")

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        return cls(
            raw_data=resource,
            state=resource["State"]["Name"],
            vpc_id=resource.get("VpcId", "no vpc id"),
            subnet_id=resource.get("SubnetId", "no subnet id"),
            id=resource["InstanceId"],
            id_ng=resource["InstanceId"],
            name=cls.get_tags(resource).get("Name", "No instance name"),
            inst_arn=arns.res.Ec2Instance.new(
                aws_account_id=bsm.aws_account_id,
                aws_region=bsm.aws_region,
                resource_id=resource["InstanceId"],
            ).to_arn(),
        )

    @property
    def title(self) -> str:
        return rl.format_key_value("name_tag", self.name)

    @property
    def subtitle(self) -> str:
        return "{} | {} | {} | {}, {}".format(
            f"{self.state_icon} {self.state}",
            rl.highlight_text(self.id),
            self.inst_type,
            f"vpc = {self.vpc_id}",
            self.short_subtitle,
        )

    @property
    def autocomplete(self) -> str:
        return self.id

    @property
    def arn(self) -> str:
        return self.inst_arn

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.ec2.get_instance(instance_id_or_arn=self.arn)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.ec2.instances

    # fmt: off
    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        from_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.ec2_client.describe_instances(InstanceIds=[self.id])
            instances = res.get("Reservations", [{}])[0].get("Instances", [])
            if len(instances) == 0:
                return [
                    rl.DetailItem.new(
                        title="🚨 EC2 instance not found, maybe it's deleted?",
                        subtitle=f"{rl.ShortcutEnum.ENTER} to verify in AWS Console",
                        url=url,
                    )
                ]
            inst = self.from_resource(
                instances[0], ars.bsm, dict(InstanceIds=[self.id])
            )
            detail_items.extend([
                from_detail("inst_id", inst.id, url=url),
                from_detail("inst_type", inst.inst_type, url=url),
                from_detail("state", inst.state, value_text=f"{self.state_icon} {inst.state}", url=url),
                from_detail("vpc_id", inst.vpc_id, url=ars.aws_console.vpc.get_vpc(inst.vpc_id)),
                from_detail("subnet_id", inst.subnet_id, url=ars.aws_console.vpc.get_subnet(inst.subnet_id)),
                from_detail("public_ip", inst.public_ip, url=url),
                from_detail("private_ip", inst.private_ip, url=url),
                from_detail("platform", inst.platform, url=url),
                from_detail("inst_profile", inst.inst_profile_arn, url=ars.aws_console.iam.get_role(inst.inst_profile_arn.split("/")[-1])),
            ])
            detail_items.extend(
                [
                    from_detail(
                        "sg",
                        dct["GroupId"],
                        value_text="{} | {}".format(dct["GroupId"], dct["GroupName"]),
                        url=ars.aws_console.vpc.get_security_group(dct["GroupId"]),
                        uid=dct["GroupId"],
                    )
                    for dct in inst.raw_data.get("SecurityGroups", [])
                ]
            )
            detail_items.extend(rl.DetailItem.from_tags(inst.tags, url=url))
        return detail_items
    # fmt: on


class Ec2InstanceSearcher(rl.BaseSearcher[Ec2Instance]):
    """
    todo: docstring
    """
    pass


ec2_instance_searcher = Ec2InstanceSearcher(
    # list resources
    service="ec2",
    method="describe_instances",
    is_paginator=True,
    default_boto_kwargs={"PaginationConfig": {"MaxItems": 9999, "PageSize": 1000}},
    result_path=rl.ResultPath("Reservations[].Instances[]"),
    # extract document
    doc_class=Ec2Instance,
    # search
    resource_type=rl.SearcherEnum.ec2_instance.value,
    fields=Ec2Instance.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(rl.SearcherEnum.ec2_instance.value),
    more_cache_key=None,
)


ec2_vpc_state_icon_mapper = {
    "pending": "🟡",
    "available": "🟢",
}


@dataclasses.dataclass
class Ec2Vpc(rl.ResourceDocument, Ec2Mixin):
    # fmt: off
    state: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="state", minsize=2, maxsize=4, stored=True)})
    cidr_ipv4: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="cidr_ipv4", minsize=2, maxsize=3, stored=True)})
    cidr_ipv6: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="cidr_ipv6", minsize=2, maxsize=4, stored=True)})
    id_ng: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="id_ng", minsize=2, maxsize=4, stored=True)})
    vpc_arn: str = dataclasses.field(metadata={"field": sayt.StoredField(name="vpc_arn")})
    # fmt: on

    @property
    def is_default(self) -> str:
        return self.raw_data["IsDefault"]

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        CidrBlockAssociationSet = resource.get("CidrBlockAssociationSet", [])
        if CidrBlockAssociationSet:
            cidr_ipv4 = CidrBlockAssociationSet[0].get("CidrBlock", "NA")
        else:
            cidr_ipv4 = "NA"

        Ipv6CidrBlockAssociationSet = resource.get("Ipv6CidrBlockAssociationSet", [])
        if Ipv6CidrBlockAssociationSet:
            cidr_ipv6 = Ipv6CidrBlockAssociationSet[0].get("Ipv6CidrBlock", "NA")
        else:
            cidr_ipv6 = "NA"

        return cls(
            raw_data=resource,
            state=resource["State"],
            cidr_ipv4=cidr_ipv4,
            cidr_ipv6=cidr_ipv6,
            id=resource["VpcId"],
            id_ng=resource["VpcId"],
            name=cls.get_tags(resource).get("Name", "No vpc name"),
            vpc_arn=arns.res.Vpc.new(
                aws_account_id=bsm.aws_account_id,
                aws_region=bsm.aws_region,
                resource_id=resource["VpcId"],
            ).to_arn(),
        )

    @property
    def state_icon(self) -> str:
        return ec2_vpc_state_icon_mapper[self.state]

    @property
    def title(self) -> str:
        return rl.format_key_value("name_tag", self.name)

    @property
    def subtitle(self) -> str:
        return "{} | {} | {}, {}".format(
            f"{self.state_icon} {self.state}",
            rl.highlight_text(self.id),
            rl.format_key_value("is_default", self.is_default),
            self.short_subtitle,
        )

    @property
    def autocomplete(self) -> str:
        return self.id

    @property
    def arn(self) -> str:
        return self.vpc_arn

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.vpc.get_vpc(vpc_id=self.id)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.vpc.vpcs

    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        get_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.ec2_client.describe_vpcs(VpcIds=[self.id])
            vpcs = res.get("Vpcs", [])
            if len(vpcs) == 0:
                return [
                    rl.DetailItem.new(
                        title="🚨 Ec2 vpc not found, maybe it's deleted?",
                        subtitle=f"{rl.ShortcutEnum.ENTER} to verify in AWS Console",
                        url=url,
                    )
                ]
            vpc = self.from_resource(vpcs[0], ars.bsm, dict(VpcIds=[self.id]))
            # fmt: off
            detail_items.extend(
                [
                    get_detail("vpc_id", vpc.id, url=url),
                    get_detail("is_default", vpc.is_default, url=url),
                    get_detail("state", vpc.state, f"{vpc.state_icon} {vpc.state}", url=url),
                    get_detail("cidr_ipv4", vpc.cidr_ipv4, url=url),
                    get_detail("cidr_ipv6", vpc.cidr_ipv6, url=url),
                ]
            )
            # fmt: on
            detail_items.extend(rl.DetailItem.from_tags(vpc.tags, url))
        return detail_items


class Ec2VpcSearcher(rl.BaseSearcher[Ec2Vpc]):
    pass


ec2_vpc_searcher = Ec2VpcSearcher(
    # list resources
    service="ec2",
    method="describe_vpcs",
    is_paginator=True,
    default_boto_kwargs={"PaginationConfig": {"MaxItems": 9999, "PageSize": 1000}},
    result_path=rl.ResultPath("Vpcs"),
    # extract document
    doc_class=Ec2Vpc,
    # search
    resource_type=rl.SearcherEnum.ec2_vpc.value,
    fields=Ec2Vpc.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(rl.SearcherEnum.ec2_vpc.value),
    more_cache_key=None,
)


@dataclasses.dataclass
class Ec2Subnet(rl.ResourceDocument, Ec2Mixin):
    # fmt: off
    state: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="state", minsize=2, maxsize=4, stored=True)})
    vpc_id: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="vpc_id", minsize=2, maxsize=4, stored=True)})
    az: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="az", minsize=2, maxsize=4, stored=True)})
    id_ng: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="id_ng", minsize=2, maxsize=4, stored=True)})
    # fmt: on

    @property
    def state_icon(self) -> str:
        return ec2_vpc_state_icon_mapper[self.state]

    @property
    def available_ip(self) -> str:
        return self.raw_data.get("AvailableIpAddressCount", "NA")

    @property
    def enable_dns64(self) -> str:
        return self.raw_data.get("EnableDns64", "NA")

    @property
    def ipv6_native(self) -> str:
        return self.raw_data.get("Ipv6Native", "NA")

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        return cls(
            raw_data=resource,
            state=resource["State"],
            vpc_id=resource["VpcId"],
            az=resource["AvailabilityZone"],
            id=resource["SubnetId"],
            id_ng=resource["SubnetId"],
            name=cls.get_tags(resource).get("Name", "No subnet name"),
        )

    @property
    def title(self) -> str:
        return rl.format_key_value("name_tag", self.name)

    @property
    def subtitle(self) -> str:
        state_icon = ec2_vpc_state_icon_mapper[self.state]
        return "{} | {} | {} | {}, {}".format(
            f"{state_icon} {self.state}",
            self.vpc_id,
            rl.highlight_text(self.id),
            self.az,
            self.short_subtitle,
        )

    @property
    def autocomplete(self) -> str:
        return self.id

    @property
    def arn(self) -> str:
        return self.raw_data["SubnetArn"]

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.vpc.get_subnet(subnet_id=self.id)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.vpc.subnets

    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        from_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.ec2_client.describe_subnets(SubnetIds=[self.id])
            subnets = res.get("Subnets", [])
            if len(subnets) == 0:
                return [
                    rl.DetailItem.new(
                        title="🚨 EC2 subnet not found, maybe it's deleted?",
                        subtitle=f"{rl.ShortcutEnum.ENTER} to verify in AWS Console",
                        url=url,
                    )
                ]
            subnet = self.from_resource(subnets[0], ars.bsm, dict(SubnetIds=[self.id]))
            # fmt: off
            detail_items.extend([
                from_detail("subnet_id", subnet.id, url=url),
                from_detail("vpc_id", subnet.vpc_id, url=ars.aws_console.vpc.get_vpc(subnet.vpc_id)),
                from_detail("az", subnet.az),
                from_detail("state", subnet.state, f"{subnet.state_icon} {subnet.state}", url=url),
                from_detail("available_ip", subnet.available_ip, url=url),
                from_detail("enable_dns64", subnet.enable_dns64, url=url),
                from_detail("ipv6_native", subnet.ipv6_native, url=url),
            ])
            # fmt: on
            detail_items.extend(rl.DetailItem.from_tags(subnet.tags, url))
        return detail_items


class Ec2SubnetSearcher(rl.BaseSearcher[Ec2Subnet]):
    pass


ec2_subnet_searcher = Ec2SubnetSearcher(
    # list resources
    service="ec2",
    method="describe_subnets",
    is_paginator=True,
    default_boto_kwargs={"PaginationConfig": {"MaxItems": 9999, "PageSize": 1000}},
    result_path=rl.ResultPath("Subnets"),
    # extract document
    doc_class=Ec2Subnet,
    # search
    resource_type=rl.SearcherEnum.ec2_subnet.value,
    fields=Ec2Subnet.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(rl.SearcherEnum.ec2_subnet.value),
    more_cache_key=None,
)


@dataclasses.dataclass
class Ec2SecurityGroup(rl.ResourceDocument, Ec2Mixin):
    # fmt: off
    vpc_id: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="vpc_id", minsize=2, maxsize=4, stored=True)})
    id_ng: str = dataclasses.field(metadata={"field": sayt.NgramWordsField(name="id_ng", minsize=2, maxsize=4, stored=True)})
    sg_arn: str = dataclasses.field(metadata={"field": sayt.StoredField(name="sg_arn")})
    # fmt: on

    @property
    def description(self) -> str:
        return rl.get_description(self.raw_data, "Description")

    @classmethod
    def from_resource(cls, resource, bsm, boto_kwargs):
        return cls(
            raw_data=resource,
            vpc_id=resource["VpcId"],
            id=resource["GroupId"],
            id_ng=resource["GroupId"],
            name=resource.get("GroupName", "NA"),
            sg_arn=arns.res.SecurityGroup.new(
                aws_account_id=bsm.aws_account_id,
                aws_region=bsm.aws_region,
                resource_id=resource["GroupId"],
            ).to_arn(),
        )

    @property
    def title(self) -> str:
        return rl.format_key_value("group_name", self.name)

    @property
    def subtitle(self) -> str:
        return "{} | {} | {}, {}".format(
            self.vpc_id,
            rl.highlight_text(self.id),
            self.description,
            self.short_subtitle,
        )

    @property
    def autocomplete(self) -> str:
        return self.id

    @property
    def arn(self) -> str:
        return self.sg_arn

    def get_console_url(self, console: acu.AWSConsole) -> str:
        return console.vpc.get_security_group(sg_id=self.id)

    @classmethod
    def get_list_resources_console_url(cls, console: acu.AWSConsole) -> str:
        return console.vpc.security_groups

    def get_details(self, ars: "ARS") -> T.List[rl.DetailItem]:
        from_detail = rl.DetailItem.from_detail
        url = self.get_console_url(console=ars.aws_console)
        detail_items = rl.DetailItem.get_initial_detail_items(doc=self, ars=ars)

        with rl.DetailItem.error_handling(detail_items):
            res = ars.bsm.ec2_client.describe_security_groups(GroupIds=[self.id])
            sgs = res.get("SecurityGroups", [])
            if len(sgs) == 0:
                return [
                    rl.DetailItem.new(
                        title="🚨 EC2 security group not found, maybe it's deleted?",
                        subtitle=f"{rl.ShortcutEnum.ENTER} to verify in AWS Console",
                        url=url,
                    )
                ]
            sg = self.from_resource(sgs[0], ars.bsm, dict(GroupIds=[self.id]))
            # fmt: off
            detail_items.extend([
                from_detail("sg_id", sg.id, url=url),
                from_detail("description", sg.description, url=url),
                from_detail("vpc_id", sg.vpc_id, url=ars.aws_console.vpc.get_vpc(sg.vpc_id)),
            ])
            # fmt: on
            detail_items.extend(rl.DetailItem.from_tags(sg.tags, url))
        return detail_items


class Ec2SecurityGroupSearcher(rl.BaseSearcher[Ec2SecurityGroup]):
    pass


ec2_securitygroup_searcher = Ec2SecurityGroupSearcher(
    # list resources
    service="ec2",
    method="describe_security_groups",
    is_paginator=True,
    default_boto_kwargs={"PaginationConfig": {"MaxItems": 9999, "PageSize": 1000}},
    result_path=rl.ResultPath("SecurityGroups"),
    # extract document
    doc_class=Ec2SecurityGroup,
    # search
    resource_type=rl.SearcherEnum.ec2_security_group.value,
    fields=Ec2SecurityGroup.get_dataset_fields(),
    cache_expire=rl.config.get_cache_expire(rl.SearcherEnum.ec2_security_group.value),
    more_cache_key=None,
)



# -*- coding: utf-8 -*-

from aws_resource_search.handlers.search_resource_type_handler import (
    search_resource_type_and_return_items,
)
from aws_resource_search.tests.fake_aws.main import FakeAws


class TestSearchResourceTypeHandler(FakeAws):
    @classmethod
    def setup_class_post_hook(cls):
        cls.setup_ars()
        cls.setup_ui()

    def test_search_resource_type_and_return_items(self):
        for query in [
            "s3",
            "iam",
            "sfn",
        ]:
            items = search_resource_type_and_return_items(ui=self.ui, query=query)
            for item in items:
                assert query in item.title


if __name__ == "__main__":
    from aws_resource_search.tests.helper import run_cov_test

    run_cov_test(
        __file__,
        "aws_resource_search.handlers.search_resource_type_handler",
        preview=False,
    )



# -*- coding: utf-8 -*-

"""
See :func:`select_resource_type_handler`.
"""

import typing as T
import json

import zelfred.api as zf
import sayt.api as sayt

from ..paths import dir_index, dir_cache, path_searchers_json
from .. import res_lib as rl


if T.TYPE_CHECKING:  # pragma: no cover
    from ..ui_def import UI


def downloader() -> T.List[sayt.T_DOCUMENT]:
    data = json.loads(path_searchers_json.read_text())
    return [
        rl.ResourceTypeDocument(
            id=resource_type,
            name=resource_type,
            desc=dct["desc"],
            ngram=dct["ngram"],
        ).to_dict()
        for resource_type, dct in data.items()
    ]


index_name = "aws-resource-search-resource-types"
resource_type_dataset = sayt.DataSet(
    dir_index=dir_index,
    index_name=index_name,
    fields=[
        sayt.IdField(name="id", stored=True),
        sayt.NgramWordsField(name="name", stored=True, minsize=2, maxsize=10),
        sayt.StoredField(name="desc"),
        sayt.NgramWordsField(name="ngram", stored=True, minsize=2, maxsize=10),
    ],
    dir_cache=dir_cache,
    cache_key=index_name,
    cache_tag=index_name,
    cache_expire=24 * 60 * 60,
    downloader=downloader,
)


def search_resource_type_and_return_items(
    ui: "UI",
    query: str,
    refresh_data: bool = False,
) -> T.List[T.Union[rl.AwsResourceTypeItem, rl.UrlItem]]:
    """
    Search AWS Resource Type based on the query and return items.
    """
    docs = resource_type_dataset.search(
        query=query,
        limit=50,
        simple_response=True,
        refresh_data=refresh_data,
    )
    if len(docs):
        return rl.AwsResourceTypeItem.from_many_document(
            docs=[rl.ResourceTypeDocument.from_dict(doc) for doc in docs],
            ars=ui.ars,
        )
    else:
        return [
            rl.UrlItem.from_url(
                url="https://github.com/MacHu-GWU/aws_resource_search-project/issues/new?assignees=MacHu-GWU&labels=enhancement&projects=&template=support-new-aws-resource.md&title=%5BFeature%5D+I+want+to+be+able+to+search+%24%7Bservice_name%7D-%24%7Bresource_name%7D",
                title=f"🔴 No resource type found, maybe it is not supported yet.",
                subtitle=(
                    "Please try another query, "
                    "or type {} to submit a Github issue for new resource, "
                    "or tap {} to clear existing query."
                ).format(
                    rl.ShortcutEnum.ENTER,
                    rl.ShortcutEnum.TAB,
                ),
                autocomplete="",
            )
        ]


def search_resource_type_handler(
    ui: "UI",
    query: str,
    skip_ui: bool = False,
) -> T.List[T.Union[rl.AwsResourceTypeItem, rl.UrlItem]]:  # pragma: no cover
    """
    Search AWS resource type defined in the ``aws_resource_search/searchers.json`` file.
    Afterwards, user can start typing query to filter AWS resources of given type.

    :param query: service id and resource type search query input. For example:
        "s3 bucket", "ec2 inst"
    :param skip_ui: if True, skip the UI related logic, just return the items.
        this argument is used for third party integration.
    """
    if skip_ui is False:  # pragma: no cover
        ui.render.prompt = "(Resource Type)"
    final_query = rl.preprocess_query(query)

    # manually refresh data
    # show "loading" message
    if query.strip().endswith("!~"):
        if skip_ui is False:  # pragma: no cover
            ui.run_handler(items=[])
            ui.repaint()
            ui.line_editor.press_backspace(n=2)
        return search_resource_type_and_return_items(
            ui=ui,
            query=rl.preprocess_query(final_query[:-2]),
            refresh_data=True,
        )

    # example: "ec2 inst"
    items = search_resource_type_and_return_items(
        ui=ui,
        query=final_query,
        refresh_data=False,
    )
    zf.debugger.log(f"end of search_resource_type_handler")
    zf.debugger.log(
        f"ui.dropdown.selected_item_index = {ui.dropdown.selected_item_index}"
    )
    return items



# -*- coding: utf-8 -*-

"""
See :class:`ShowAwsInfoItem`.
"""

import typing as T
import dataclasses

from ..terminal import format_key_value, ShortcutEnum
from .base_item import BaseArsItem

if T.TYPE_CHECKING:  # pragma: no cover
    from ..ui_def import UI


@dataclasses.dataclass
class ShowAwsInfoItem(BaseArsItem):
    """
    Represent an item to display current AWS Account information
    """

    def enter_handler(self, ui: "UI"):  # pragma: no cover
        pass

    def post_enter_handler(self, ui: "UI"):  # pragma: no cover
        """
        Do "autocomplete".
        """
        ui.line_editor.clear_line()
        ui.line_editor.enter_text(self.autocomplete)

    def ctrl_w_handler(self, ui: "UI"):  # pragma: no cover
        pass

    def post_ctrl_w_handler(self, ui: "UI"):  # pragma: no cover
        """
        Do "autocomplete".
        """
        ui.line_editor.clear_line()
        ui.line_editor.enter_text(self.autocomplete + "!@")

    @classmethod
    def from_key_value(
        cls,
        key: str,
        value: str,
        autocomplete: str,
    ):
        """
        Factory method for :class:`ShowAwsInfoItem`.

        :param key: the name of the aws account detail
        :param value: the value of the aws account detail
        :param autocomplete: the existing query input before "!?" to recover to.
        """
        return cls(
            title=f"Current {format_key_value(key, value)}",
            subtitle=(
                f"Hit {ShortcutEnum.TAB} or {ShortcutEnum.ENTER} to go back, "
                f"Hit {ShortcutEnum.CTRL_W} to pick another AWS profile"
            ),
            uid=f"aws-account-info-{key}",
            autocomplete=autocomplete,
        )



# -*- coding: utf-8 -*-

from .vendor.better_enum import BetterStrEnum


class SearcherEnum(BetterStrEnum):
    cloudformation_stack = "cloudformation-stack"
    codebuild_job_run = "codebuild-job-run"
    codebuild_project = "codebuild-project"
    codecommit_repository = "codecommit-repository"
    codepipeline_pipeline = "codepipeline-pipeline"
    dynamodb_table = "dynamodb-table"
    ec2_instance = "ec2-instance"
    ec2_security_group = "ec2-security-group"
    ec2_subnet = "ec2-subnet"
    ec2_vpc = "ec2-vpc"
    ecr_repository = "ecr-repository"
    ecr_repository_image = "ecr-repository-image"
    ecs_cluster = "ecs-cluster"
    ecs_task_run = "ecs-task-run"
    ecs_task_definition_family = "ecs_task_definition_family"
    glue_crawler = "glue-crawler"
    glue_database = "glue-database"
    glue_database_table = "glue-database-table"
    glue_job = "glue-job"
    glue_job_run = "glue-job-run"
    iam_group = "iam-group"
    iam_policy = "iam-policy"
    iam_role = "iam-role"
    iam_user = "iam-user"
    kms_key_alias = "kms-key-alias"
    lambda_function = "lambda-function"
    lambda_function_alias = "lambda-function-alias"
    lambda_layer = "lambda-layer"
    rds_db_cluster = "rds-db-cluster"
    rds_db_instance = "rds-db-instance"
    s3_bucket = "s3-bucket"
    secretsmanager_secret = "secretsmanager-secret"
    sfn_state_machine = "sfn-state-machine"
    sfn_state_machine_execution = "sfn-state-machine-execution"
    sns_topic = "sns-topic"
    sqs_queue = "sqs-queue"
    ssm_parameter = "ssm-parameter"


# -*- coding: utf-8 -*-

from .open_config_handler import open_config_handler
from .search_aws_profile_handler import search_aws_profile_handler
from .search_resource_type_handler import search_resource_type_handler
from .search_resource_handler import search_resource_handler
from .show_aws_info_handler import show_aws_info_handler



# -*- coding: utf-8 -*-

from aws_resource_search.items.file_item import FileItem
from aws_resource_search.handlers.open_config_handler import (
    open_config_handler,
)
from aws_resource_search.tests.fake_aws.main import FakeAws


class Test(FakeAws):
    @classmethod
    def setup_class_post_hook(cls):
        cls.setup_ars()
        cls.setup_ui()

    def test_open_config_handler(self):
        """
        We don't test handler.
        """
        items = open_config_handler(ui=self.ui, line_input="")
        assert len(items) == 1
        assert isinstance(items[0], FileItem)

        # ------------------------------------------------------------------
        # We only test this action manually on developer's machine, not in CI.
        # ------------------------------------------------------------------
        # items[0].enter_handler(ui=self.ui)


if __name__ == "__main__":
    from aws_resource_search.tests.helper import run_cov_test

    run_cov_test(
        __file__,
        "aws_resource_search.handlers.open_config_handler",
        preview=False,
    )



# -*- coding: utf-8 -*-

import typing as T
import random

from .utils import guid, envs, rand_env

if T.TYPE_CHECKING:
    from .main import FakeAws


class Ec2Mixin:
    ec2_inst_id_list: T.List[str] = list()

    @classmethod
    def create_ec2_instances(cls: T.Type["FakeAws"]):
        image_id = cls.bsm.ec2_client.describe_images()["Images"][0]["ImageId"]

        for ith in range(1, 1 + 10):
            kwargs = dict(
                MinCount=1,
                MaxCount=1,
                ImageId=image_id,
            )
            if random.randint(1, 100) <= 70:
                env = rand_env()
                name = f"{env}-{guid}-{ith}-ec2-instance"
                kwargs["TagSpecifications"] = [
                    dict(
                        ResourceType="instance",
                        Tags=[dict(Key="Name", Value=name)],
                    )
                ]
            res = cls.bsm.ec2_client.run_instances(**kwargs)
            inst_id = res["Instances"][0]["InstanceId"]
            cls.ec2_inst_id_list.append(inst_id)



# -*- coding: utf-8 -*-

"""
See :class:`ExceptionItem`.
"""

import typing as T
import traceback
import dataclasses

import zelfred.api as zf

from ..terminal import ShortcutEnum
from ..compat import TypedDict
from ..paths import path_exception_item_txt
from .base_item import BaseArsItem


class T_EXCEPTION_ITEM_VARIABLES(TypedDict):
    error: Exception
    traceback: str


@dataclasses.dataclass
class ExceptionItem(BaseArsItem):
    """
    Represent a Python exception detail in the dropdown menu.
    """

    variables: T_EXCEPTION_ITEM_VARIABLES = dataclasses.field(default_factory=dict)

    def enter_handler(self, ui: "zf.UI"):  # pragma: no cover
        """
        Behavior:

        - If we have an exception, we write the traceback data
            ``path_exception_item_txt`` and open it in the default app.
        - If not, nothing happens (most likely NOT)
        """
        if self.variables.get("error") is not None:
            path_exception_item_txt.write_text(self.variables["traceback"])
            self.open_file_or_print(ui, path_exception_item_txt)
        else:
            pass

    @classmethod
    def from_error(
        cls,
        error: Exception,
        title: T.Optional[str] = None,
        subtitle: T.Optional[str] = None,
        uid: T.Optional[str] = None,
        arg: T.Optional[str] = None,
        autocomplete: T.Optional[str] = None,
    ):
        """
        Create one :class:`ExceptionItem` from an ``Exception`` object.
        """
        if title is None:
            title = f"❗ {error!r}"
        if subtitle is None:
            subtitle = f"📄 {ShortcutEnum.ENTER} to view error details"
        kwargs = dict(
            title=title,
            subtitle=subtitle,
            arg=arg,
            autocomplete=autocomplete,
            variables={"error": error, "traceback": traceback.format_exc(limit=50)},
        )
        if uid:
            kwargs["uid"] = uid
        return cls(**kwargs)



# -*- coding: utf-8 -*-

import pytest
import moto

from aws_resource_search.tests.mock_test import BaseMockTest
from aws_resource_search.base_searcher import preprocess_query
from aws_resource_search.res.s3 import s3_bucket_searcher
from aws_resource_search.res.iam import iam_group_searcher


def test_preprocess_query():
    assert preprocess_query(None) == "*"
    assert preprocess_query("") == "*"
    assert preprocess_query("*") == "*"
    assert preprocess_query("abc") == "abc~1"
    assert preprocess_query("abc~2") == "abc~2"
    assert preprocess_query("abc xyz") == "abc~1 xyz~1"
    assert preprocess_query("abc~2 xyz") == "abc~2 xyz~1"
    assert preprocess_query("a") == "*"
    assert preprocess_query("a b c xyz") == "xyz~1"
    assert preprocess_query("s3.put_obj") == "s3~1 put~1 obj~1"
    assert preprocess_query("s*") == "s*~1"
    assert preprocess_query("s?") == "s?~1"


class TestSearcher(BaseMockTest):
    mock_list = [
        moto.mock_s3,
        moto.mock_iam,
        moto.mock_sts,
    ]

    def _create_test_buckets(self):
        self.bucket_names = [
            "company-data",
            "enterprise-data",
        ]
        for bucket_name in self.bucket_names:
            self.bsm.s3_client.create_bucket(Bucket=bucket_name)

    def _create_test_iam_groups(self):
        self.iam_group_names = [
            "admin_group",
            "developer_group",
        ]
        for iam_group_name in self.iam_group_names:
            try:
                self.bsm.iam_client.create_group(GroupName=iam_group_name)
            except Exception as e:
                if "already exists" in str(e):
                    pass
                else:
                    raise e

    def _test_get_bsm(self):
        with pytest.raises(TypeError):
            s3_bucket_searcher._get_bsm(bsm="bsm")
        s3_bucket_searcher._get_bsm(self.bsm)
        s3_bucket_searcher.bsm = self.bsm
        s3_bucket_searcher._get_bsm(self.bsm)

    def _test_search(self):
        self._create_test_buckets()
        s3_bucket_searcher.bsm = self.bsm
        res = s3_bucket_searcher.search(refresh_data=True)
        assert len(res) == 2

        res = s3_bucket_searcher.search(simple_response=False)
        assert len(res["hits"]) == 2

        self._create_test_iam_groups()
        iam_group_searcher.bsm = self.bsm
        res = iam_group_searcher.search(refresh_data=True)
        assert len(res) == 2

    def test(self):
        self._test_get_bsm()
        self._test_search()


if __name__ == "__main__":
    from aws_resource_search.tests.helper import run_cov_test

    run_cov_test(__file__, "aws_resource_search.base_searcher", preview=False)



# -*- coding: utf-8 -*-

from .base_item import BaseArsItem
from .base_item import T_ARS_ITEM
from .detail_item import DetailItem
from .exception_item import ExceptionItem
from .file_item import FileItem
from .info_item import InfoItem
from .url_item import UrlItem
from .aws_resource_type_item import AwsResourceTypeItem
from .aws_resource_item import AwsResourceItem
from .set_aws_profile_item import SetAwsProfileItem
from .show_aws_info_item import ShowAwsInfoItem



# -*- coding: utf-8 -*-

from aws_resource_search.items.show_aws_info_item import ShowAwsInfoItem


class TestShowAwsInfoItem:
    def test_from_key_value(self):
        item = ShowAwsInfoItem.from_key_value(
            "aws_account_id",
            "123456789012",
            autocomplete="",
        )
        # print(item)


if __name__ == "__main__":
    from aws_resource_search.tests.helper import run_cov_test

    run_cov_test(
        __file__,
        "aws_resource_search.items.show_aws_info_item",
        preview=False,
    )



# -*- coding: utf-8 -*-

"""
See :class:`SetAwsProfileItem`.
"""

import typing as T
import dataclasses

import awscli_mate.api as awscli_mate
import aws_console_url.api as aws_console_url

from ..terminal import ShortcutEnum, format_key_value, highlight_text
from .base_item import BaseArsItem

if T.TYPE_CHECKING:  # pragma: no cover
    from ..ars_def import ARS
    from ..ui.main import UI


def set_profile_in_bsm(profile: str, ars: "ARS"):
    """
    Update the singleton ``bsm`` object by reset its cache and use a new AWS profile.
    Also update all of related cached objects in ``ars``.
    """
    ars.bsm.aws_access_key_id = None
    ars.bsm.aws_secret_access_key = None
    ars.bsm.aws_session_token = None
    ars.bsm.region_name = None
    ars.bsm.botocore_session = None
    ars.bsm.profile_name = profile
    ars.bsm.clear_cache()

    ars.aws_console = aws_console_url.AWSConsole.from_bsm(ars.bsm)
    ars.searcher_finder.searcher_cache.clear()


@dataclasses.dataclass
class SetAwsProfileItem(BaseArsItem):
    """
    Represent an item in the aws profile search result.
    """

    def enter_handler(self, ui: "UI"):  # pragma: no cover
        awscli_mate.AWSCliConfig().set_profile_as_default(profile=self.arg)
        set_profile_in_bsm(self.arg, ui.ars)

    def post_enter_handler(self, ui: "UI"):  # pragma: no cover
        """
        When exiting the switch profile session, recover the original query input.
        """
        ui.line_editor.clear_line()
        ui.line_editor.enter_text(self.autocomplete)

    @classmethod
    def from_profile_region(
        cls,
        profile: str,
        region: str,
        autocomplete: str,
    ):
        """
        Factory method for :class:`SetAwsProfileItem`.

        :param profile: aws profile name
        :param region: aws region
        :param autocomplete: the existing query input before "!@" to recover to.
        """
        return cls(
            title="📝 {} | {}".format(
                format_key_value("profile", profile), format_key_value("region", region)
            ),
            subtitle=f"Hit {ShortcutEnum.ENTER} to set {highlight_text(profile)} as the default profile.",
            uid=f"aws-profile-{profile}",
            arg=profile,
            autocomplete=autocomplete,
        )

    @classmethod
    def from_many_profile_region_pairs(
        cls,
        pairs: T.List[T.Tuple[str, str]],
        autocomplete: str,
    ):
        """
        Another factory method to create many :class:`SetAwsProfileItem` at once.
        """
        return [
            cls.from_profile_region(profile, region, autocomplete)
            for profile, region in pairs
        ]



# -*- coding: utf-8 -*-

import typing as T

from .utils import guid, envs, rand_env

if T.TYPE_CHECKING:
    from .main import FakeAws


class S3Mixin:
    s3_buckets: T.List[str] = list()

    @classmethod
    def create_s3_bucket(cls: T.Type["FakeAws"]):
        for ith in range(1, 1 + 10):
            env = rand_env()
            bucket = f"{env}-{guid}-{ith}-s3-bucket"
            cls.bsm.s3_client.create_bucket(Bucket=bucket)
            cls.s3_buckets.append(bucket)



# -*- coding: utf-8 -*-

"""
See :class:`ARSMixin`
"""

import typing as T

from .compat import cached_property

if T.TYPE_CHECKING:  # pragma: no cover
    from .ars_def import ARS
    from .res.cloudformation import CloudFormationStackSearcher
    from .res.codebuild import CodeBuildJobRunSearcher
    from .res.codebuild import CodeBuildProjectSearcher
    from .res.codecommit import CodeCommitRepositorySearcher
    from .res.codepipeline import CodePipelinePipelineSearcher
    from .res.dynamodb import DynamodbTableSearcher
    from .res.ec2 import Ec2InstanceSearcher
    from .res.ec2 import Ec2SecurityGroupSearcher
    from .res.ec2 import Ec2SubnetSearcher
    from .res.ec2 import Ec2VpcSearcher
    from .res.ecr import EcrRepositorySearcher
    from .res.ecr import EcrRepositoryImageSearcher
    from .res.ecs import EcsClusterSearcher
    from .res.ecs import EcsTaskRunSearcher
    from .res.ecs import EcsTaskDefinitionFamilySearcher
    from .res.glue import GlueCrawlerSearcher
    from .res.glue import GlueDatabaseSearcher
    from .res.glue import GlueTableSearcher
    from .res.glue import GlueJobSearcher
    from .res.glue import GlueJobRunSearcher
    from .res.iam import IamGroupSearcher
    from .res.iam import IamPolicySearcher
    from .res.iam import IamRoleSearcher
    from .res.iam import IamUserSearcher
    from .res.kms import KmsKeyAliasSearcher
    from .res.awslambda import LambdaFunctionSearcher
    from .res.awslambda import LambdaFunctionAliasSearcher
    from .res.awslambda import LambdaLayerSearcher
    from .res.rds import BaseSearcher
    from .res.rds import RdsDbInstanceSearcher
    from .res.s3 import S3BucketSearcher
    from .res.secretmanager import SecretsManagerSecretSearcher
    from .res.sfn import SfnStateMachineSearcher
    from .res.sfn import SfnExecutionSearcher
    from .res.sns import SnsTopicSearcher
    from .res.sqs import SqsQueueSearcher
    from .res.ssm import SsmParameterSearcher


class ARSMixin:  # pragma: no cover
    @cached_property
    def cloudformation_stack(self: "ARS") -> "CloudFormationStackSearcher":
        return self.get_searcher("cloudformation-stack")
    
    @cached_property
    def codebuild_job_run(self: "ARS") -> "CodeBuildJobRunSearcher":
        return self.get_searcher("codebuild-job-run")
    
    @cached_property
    def codebuild_project(self: "ARS") -> "CodeBuildProjectSearcher":
        return self.get_searcher("codebuild-project")
    
    @cached_property
    def codecommit_repository(self: "ARS") -> "CodeCommitRepositorySearcher":
        return self.get_searcher("codecommit-repository")
    
    @cached_property
    def codepipeline_pipeline(self: "ARS") -> "CodePipelinePipelineSearcher":
        return self.get_searcher("codepipeline-pipeline")
    
    @cached_property
    def dynamodb_table(self: "ARS") -> "DynamodbTableSearcher":
        return self.get_searcher("dynamodb-table")
    
    @cached_property
    def ec2_instance(self: "ARS") -> "Ec2InstanceSearcher":
        return self.get_searcher("ec2-instance")
    
    @cached_property
    def ec2_security_group(self: "ARS") -> "Ec2SecurityGroupSearcher":
        return self.get_searcher("ec2-security-group")
    
    @cached_property
    def ec2_subnet(self: "ARS") -> "Ec2SubnetSearcher":
        return self.get_searcher("ec2-subnet")
    
    @cached_property
    def ec2_vpc(self: "ARS") -> "Ec2VpcSearcher":
        return self.get_searcher("ec2-vpc")
    
    @cached_property
    def ecr_repository(self: "ARS") -> "EcrRepositorySearcher":
        return self.get_searcher("ecr-repository")
    
    @cached_property
    def ecr_repository_image(self: "ARS") -> "EcrRepositoryImageSearcher":
        return self.get_searcher("ecr-repository-image")
    
    @cached_property
    def ecs_cluster(self: "ARS") -> "EcsClusterSearcher":
        return self.get_searcher("ecs-cluster")
    
    @cached_property
    def ecs_task_run(self: "ARS") -> "EcsTaskRunSearcher":
        return self.get_searcher("ecs-task-run")
    
    @cached_property
    def ecs_task_definition_family(self: "ARS") -> "EcsTaskDefinitionFamilySearcher":
        return self.get_searcher("ecs_task_definition_family")
    
    @cached_property
    def glue_crawler(self: "ARS") -> "GlueCrawlerSearcher":
        return self.get_searcher("glue-crawler")
    
    @cached_property
    def glue_database(self: "ARS") -> "GlueDatabaseSearcher":
        return self.get_searcher("glue-database")
    
    @cached_property
    def glue_database_table(self: "ARS") -> "GlueTableSearcher":
        return self.get_searcher("glue-database-table")
    
    @cached_property
    def glue_job(self: "ARS") -> "GlueJobSearcher":
        return self.get_searcher("glue-job")
    
    @cached_property
    def glue_job_run(self: "ARS") -> "GlueJobRunSearcher":
        return self.get_searcher("glue-job-run")
    
    @cached_property
    def iam_group(self: "ARS") -> "IamGroupSearcher":
        return self.get_searcher("iam-group")
    
    @cached_property
    def iam_policy(self: "ARS") -> "IamPolicySearcher":
        return self.get_searcher("iam-policy")
    
    @cached_property
    def iam_role(self: "ARS") -> "IamRoleSearcher":
        return self.get_searcher("iam-role")
    
    @cached_property
    def iam_user(self: "ARS") -> "IamUserSearcher":
        return self.get_searcher("iam-user")
    
    @cached_property
    def kms_key_alias(self: "ARS") -> "KmsKeyAliasSearcher":
        return self.get_searcher("kms-key-alias")
    
    @cached_property
    def lambda_function(self: "ARS") -> "LambdaFunctionSearcher":
        return self.get_searcher("lambda-function")
    
    @cached_property
    def lambda_function_alias(self: "ARS") -> "LambdaFunctionAliasSearcher":
        return self.get_searcher("lambda-function-alias")
    
    @cached_property
    def lambda_layer(self: "ARS") -> "LambdaLayerSearcher":
        return self.get_searcher("lambda-layer")
    
    @cached_property
    def rds_db_cluster(self: "ARS") -> "BaseSearcher":
        return self.get_searcher("rds-db-cluster")
    
    @cached_property
    def rds_db_instance(self: "ARS") -> "RdsDbInstanceSearcher":
        return self.get_searcher("rds-db-instance")
    
    @cached_property
    def s3_bucket(self: "ARS") -> "S3BucketSearcher":
        return self.get_searcher("s3-bucket")
    
    @cached_property
    def secretsmanager_secret(self: "ARS") -> "SecretsManagerSecretSearcher":
        return self.get_searcher("secretsmanager-secret")
    
    @cached_property
    def sfn_state_machine(self: "ARS") -> "SfnStateMachineSearcher":
        return self.get_searcher("sfn-state-machine")
    
    @cached_property
    def sfn_state_machine_execution(self: "ARS") -> "SfnExecutionSearcher":
        return self.get_searcher("sfn-state-machine-execution")
    
    @cached_property
    def sns_topic(self: "ARS") -> "SnsTopicSearcher":
        return self.get_searcher("sns-topic")
    
    @cached_property
    def sqs_queue(self: "ARS") -> "SqsQueueSearcher":
        return self.get_searcher("sqs-queue")
    
    @cached_property
    def ssm_parameter(self: "ARS") -> "SsmParameterSearcher":
        return self.get_searcher("ssm-parameter")
    